package com.cj.jiaqingjiayi.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

import com.cj.jiaqingjiayi.common.BaseResponse;
import com.cj.jiaqingjiayi.common.ErrorCode;
import com.cj.jiaqingjiayi.common.ResultUtils;
import com.cj.jiaqingjiayi.exception.BusinessException;
import com.cj.jiaqingjiayi.exception.ThrowUtils;
import com.cj.jiaqingjiayi.model.CommonRequest;
import com.cj.jiaqingjiayi.model.domain.*;
import com.cj.jiaqingjiayi.model.request.cart.CartAddRequest;
import com.cj.jiaqingjiayi.model.request.cart.CartDeleteRequest;
import com.cj.jiaqingjiayi.model.request.cart.CartQueryRequest;
import com.cj.jiaqingjiayi.model.request.cart.CartUpdateRequest;
import com.cj.jiaqingjiayi.model.vo.*;
import com.cj.jiaqingjiayi.service.BusinessService;
import com.cj.jiaqingjiayi.service.CartService;
import com.cj.jiaqingjiayi.service.CommoditiesService;
import com.cj.jiaqingjiayi.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@RestController
@Slf4j
@Api(tags = "购物车接口")
@RequestMapping("/cart")
public class CartController {

    @Resource
    private CartService cartService;

    @Resource
    private UserService userService;

    @Resource
    private BusinessService businessService;

    @Resource
    private CommoditiesService commoditiesService;

    /**
     * 添加进购物车
     * @param cartAddRequest 添加请求
     * @param request 网络请求
     * @return 购物车id
     */
    @ApiOperation(value = "添加购物车")
    @PostMapping("/add")
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse<CartVO> addCart (@RequestBody CartAddRequest cartAddRequest, HttpServletRequest request) {
        if (cartAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        //校验
        cartService.validCart(cartAddRequest);
        //添加商品到购物车
        Cart cart = cartService.addCart(cartAddRequest, request);
        CartVO cartVO = new CartVO();
        BeanUtils.copyProperties(cart, cartVO);
        Long commoditiesId = cartVO.getCommoditiesId();
        Commodities commodities = commoditiesService.getById(commoditiesId);
        List<CommoditiesVO> commoditiesList = new ArrayList<>();
        CommoditiesVO commoditiesVO = commoditiesService.getCommoditiesVO(commodities);
        commoditiesList.add(commoditiesVO);
        cartVO.setCommoditiesVO(commoditiesList);
        return ResultUtils.success(cartVO);
    }

    /**
     * 更新购物车
     * @param cartUpdateRequest 更新请求
     * @param request 网络请求
     * @return 是否成功
     */
//    @ApiOperation(value = "更新购物车")
//    @PostMapping("/update")
//    @Transactional(rollbackFor = Exception.class)
//    public BaseResponse<Boolean> updateCart (@RequestBody CartUpdateRequest cartUpdateRequest, HttpServletRequest request){
//        if (cartUpdateRequest == null) {
//            throw new BusinessException(ErrorCode.PARAMS_ERROR);
//        }
//        //校验
//        CartAddRequest cartAddRequest = new CartAddRequest();
//        BeanUtils.copyProperties(cartUpdateRequest,cartAddRequest);
//        cartService.validCart(cartAddRequest);
//
//        Boolean cart = cartService.updateCart(cartUpdateRequest, request);
//        return ResultUtils.success(cart);
//    }

    /**
     * 删除购物车
     * @param cartDeleteRequest 删除请求
     * @param request 网络请求
     * @return 是否成功
     */
    @ApiOperation(value = "删除购物车")
    @PostMapping("/delete")
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse<Boolean> deleteCart (@RequestBody CartDeleteRequest cartDeleteRequest, HttpServletRequest request) {
        if (cartDeleteRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        Boolean cart = cartService.deleteCart(cartDeleteRequest, request);
        ThrowUtils.throwIf(!cart, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }

    /**
     * 根据商户id获取购物车脱敏列表
     * @param cartQueryRequest 商户id
     * @param request 网络请求
     * @return 脱敏列表
     */
    @ApiOperation(value = "根据商户id获取购物车脱敏列表")
    @PostMapping("/selectByBusinessId")
    public BaseResponse<CartResponseVO> selectByBusinessId(@RequestBody CartQueryRequest cartQueryRequest, HttpServletRequest request) {
        if (cartQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        Long businessId = cartQueryRequest.getId();
        Long userId = cartQueryRequest.getUserId();

        // 查询购物车列表
        LambdaQueryWrapper<Cart> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Cart::getBusinessId, businessId);
        List<Cart> cartList = cartService.list(queryWrapper);

        // 查询对应的 Business 信息
        Business business = businessService.getById(businessId);
        if (business == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "商家不存在");
        }

        // 转换 Cart 数据为 CartVO
        List<CartVO> cartVOList = this.getCartVOList(cartList);

        // 封装返回对象
        CartResponseVO responseVO = new CartResponseVO();
        responseVO.setBusiness(business);
        responseVO.setCartList(cartVOList);

        return ResultUtils.success(responseVO);
    }


    /**
     * 根据用户查找购物车列表
     * @param request 网络请求
     * @return 脱敏列表
     */
    @ApiOperation(value = "根据用户查找购物车列表")
    @PostMapping("/selectByUserId")
    public BaseResponse<List<CartDTO>> selectByUserId (@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
//        User loginUser = userService.getLoginUser(request);
//        Long userId = loginUser.getId();
        Long userId = commonRequest.getId();
        LambdaQueryWrapper<Cart> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(Cart::getUserId, userId);

        List<CartDTO> cartDTOList = new ArrayList<>();

        List<Cart> cartList = cartService.list(wrapper);
        List<CartVO> cartVOList = this.getCartVOList(cartList);

        // 遍历 UserRating 列表，根据 userId 查询 User 信息
        for (CartVO cartVO : cartVOList) {
            Business business = businessService.getById(cartVO.getBusinessId());
            CartDTO cartDTO = new CartDTO();
            cartDTO.setCartVO(cartVO);
            cartDTO.setBusiness(business);
            cartDTOList.add(cartDTO);
        }

        return ResultUtils.success(cartDTOList);
    }

    /**
     * 清空商家内的购物车
     * @param cartQueryRequest 商家id
     * @param request 网络请求
     * @return 是否成功
     */
    @ApiOperation(value = "清空商家内的购物车")
    @PostMapping("/empty/business")
    public BaseResponse<Boolean> emptyBusinessCart(@RequestBody CartQueryRequest cartQueryRequest, HttpServletRequest request) {
        if (cartQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        Long businessId = cartQueryRequest.getId();
//        User loginUser = userService.getLoginUser(request);
//        Long userId = loginUser.getId();
        Long userId = cartQueryRequest.getUserId();
        LambdaQueryWrapper<Cart> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Cart::getBusinessId, businessId);
        queryWrapper.eq(Cart::getUserId, userId);
        boolean remove = cartService.remove(queryWrapper);
        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR);

        return ResultUtils.success(true);
    }

    /**
     * 清空用户购物车
     * @param request 网络请求
     * @return 是否成功
     */
    @ApiOperation(value = "清空用户购物车")
    @PostMapping("/empty/user")
    public BaseResponse<Boolean> emptyUserCart(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
//        User loginUser = userService.getLoginUser(request);
//        Long userId = loginUser.getId();
        Long userId = commonRequest.getId();
        LambdaQueryWrapper<Cart> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Cart::getUserId, userId);
        boolean remove = cartService.remove(queryWrapper);
        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR);

        return ResultUtils.success(true);
    }

    private List<CartVO> getCartVOList(List<Cart> cartList) {
        if (CollectionUtils.isEmpty(cartList)) {
            return new ArrayList<>();
        }

        return cartList.stream().map(item ->{
            CartVO cartVO = new CartVO();
            BeanUtils.copyProperties(item, cartVO);
            return cartVO;
        }).collect(Collectors.toList());
    }
}
